#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import os
import datetime
import time
import subprocess
import threading
import socket


class Comunicator(threading.Thread):
    def __init__(self, f):
        self.host = ''
        self.port = 7678
        self.ss = None
        self.func = f
        self.conn = None
        self.addr = None
        self.stop = False
        super().__init__()
    
    def run(self):
        self.ss = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.ss.bind((self.host, self.port))
        self.ss.listen()
        self.conn, self.addr = self.ss.accept()
        while not self.stop:
            data = self.conn.recv(1024)
            if data:
                self.func(data.decode('utf-8'))
            else:
                break
        self.conn.close()
        self.conn = None
        self.addr = None
        print('run completato')
    
    def comunicate(self, msg):
        if not self.conn: return
        self.conn.sendall(msg.encode('utf-8'))
    
    def stopping(self):
        if self.conn:
            self.conn.sendall('Chiusura applicazione'.encode('utf-8'))
            self.conn.close()
            self.addr = None
            self.conn = None
        self.stop = True

command = ''

def set_command(com):
    global command
    command = com


if __name__ == '__main__':
    #global command
    # rilevo la direttrice della applicazione
    my_dir = os.path.abspath(os.path.dirname(sys.argv[0]))
    # definisco il name-file di log
    log_dir = os.path.join(my_dir, 'logs')
    if not os.path.exists(log_dir) or not os.path.isdir(log_dir):
        try:
            os.makedirs(log_dir)
        except OSError:
            print('Uscita su errore')
            exit(1)
    tempo = datetime.datetime.now()
    name = (str(tempo.year) + '_' + str(tempo.month) +'_' + str(tempo.day) +
            '-' + str(tempo.hour) + '_' + str(tempo.minute) +
            '_execution.log')
    f_n_l = os.path.join(log_dir, name)
    exec_routine = [[False, 'external_01.py', None],
                    [False, 'external_02.py', None],
                    [False, 'external_03.py', None],
                    [False, 'external_04.py', None]]
    str_tmp = tempo.isoformat() + ': Avvio Ciclo interno applicazione'
    print(str_tmp)
    try:
        with open(f_n_l, 'w') as f:
            f.write(str_tmp + '\n')
    except:
        print('Uscita su errore')
        exit(1)
    c = Comunicator(set_command)
    c.start()
    while True:
        if command:
            if command == 'get log':
                command = ''
                dimfile = str(os.stat(f_n_l).st_size) + '.'
                c.comunicate(dimfile)
                try:
                    with open(f_n_l, 'r') as f:
                        data = f.read()
                    data = data
                    c.comunicate(data)
                except:
                    pass
            elif command == 'quit':
                tempo = datetime.datetime.now()
                str_tmp = tempo.isoformat() + ': Richiesta interruzione'
                print(str_tmp)
                try:
                    with open(f_n_l, 'a') as f:
                        f.write(str_tmp + '\n')
                except:
                    pass
                break
        launch = False
        for a in exec_routine:
            if not a[0]:
                tempo = datetime.datetime.now()
                str_tmp = tempo.isoformat() + ': Avvio applicazione esterna ' + a[1]
                print(str_tmp)
                try:
                    with open(f_n_l, 'a') as f:
                        f.write(str_tmp + '\n')
                except:
                    pass
                a[0] = True
                launch = True
                proc = subprocess.Popen(['python3', os.path.join(my_dir, a[1]), f_n_l])
                a[2] = proc
                break
        time.sleep(100)
        if not launch:
            break
    tempo = datetime.datetime.now()
    str_tmp = tempo.isoformat() + ': Ciclo interno completato'
    print(str_tmp)
    try:
        with open(f_n_l, 'a') as f:
            f.write(str_tmp + '\n')
    except:
        pass
    c.stopping()
    del(c)
    if command == 'quit':
        for a in exec_routine:
            r = a[2]
            if r:
                r.kill()